# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ai/h2o/featurestore/api/v1/OnlineApi.proto
# Protobuf Python Version: 4.25.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n*ai/h2o/featurestore/api/v1/OnlineApi.proto\x12\x1a\x61i.h2o.featurestore.api.v1\x1a\x1fgoogle/protobuf/timestamp.proto\"%\n\x16OnlineRetrieveResponse\x12\x0b\n\x03row\x18\x01 \x01(\t\"H\n\x16OnlineIngestionRequest\x12\r\n\x05token\x18\x03 \x01(\t\x12\x11\n\tsignature\x18\x04 \x01(\t\x12\x0c\n\x04rows\x18\x05 \x03(\t\")\n\x17OnlineIngestionResponse\x12\x0e\n\x06\x65rrors\x18\x01 \x03(\t\"R\n\x1bOnlineIngestionTokenRequest\x12\x16\n\x0e\x66\x65\x61ture_set_id\x18\x01 \x01(\t\x12\x1b\n\x13\x66\x65\x61ture_set_version\x18\x02 \x01(\t\"n\n\x1cOnlineIngestionTokenResponse\x12\r\n\x05token\x18\x01 \x01(\t\x12\x11\n\tsignature\x18\x02 \x01(\t\x12,\n\x08valid_to\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"Q\n\x1aOnlineRetrieveTokenRequest\x12\x16\n\x0e\x66\x65\x61ture_set_id\x18\x01 \x01(\t\x12\x1b\n\x13\x66\x65\x61ture_set_version\x18\x02 \x01(\t\"<\n#OnlineMLDataSetRetrieveTokenRequest\x12\x15\n\rml_dataset_id\x18\x01 \x01(\t\"m\n\x1bOnlineRetrieveTokenResponse\x12\r\n\x05token\x18\x01 \x01(\t\x12\x11\n\tsignature\x18\x02 \x01(\t\x12,\n\x08valid_to\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"w\n\x13OnlineFeatureSchema\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x11\n\tdata_type\x18\x02 \x01(\t\x12?\n\x06nested\x18\x03 \x03(\x0b\x32/.ai.h2o.featurestore.api.v1.OnlineFeatureSchema\"\xf5\x02\n\x0bOnlineToken\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12,\n\x08valid_to\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x16\n\x0e\x66\x65\x61ture_set_id\x18\x03 \x01(\t\x12!\n\x19\x66\x65\x61ture_set_major_version\x18\x04 \x01(\x05\x12\x14\n\x0cprimary_keys\x18\x05 \x03(\t\x12\x1a\n\x12time_travel_column\x18\x06 \x01(\t\x12!\n\x19time_travel_column_format\x18\x07 \x01(\t\x12\x14\n\x0ctime_to_live\x18\x08 \x01(\x05\x12\x1d\n\x15time_to_live_interval\x18\t \x01(\t\x12!\n\x19online_to_offline_enabled\x18\n \x01(\x08\x12?\n\x06schema\x18\x0b \x03(\x0b\x32/.ai.h2o.featurestore.api.v1.OnlineFeatureSchema\"\x90\x02\n\x13OnlineRetrieveToken\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12,\n\x08valid_to\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x16\n\x0e\x66\x65\x61ture_set_id\x18\x03 \x01(\t\x12!\n\x19\x66\x65\x61ture_set_major_version\x18\x04 \x01(\x05\x12P\n\x0cprimary_keys\x18\x05 \x03(\x0b\x32:.ai.h2o.featurestore.api.v1.OnlineRetrieveToken.PrimaryKey\x1a-\n\nPrimaryKey\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x11\n\tdata_type\x18\x02 \x01(\t\"\x8a\x0c\n\x1cOnlineMLDatasetRetrieveToken\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12,\n\x08valid_to\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x15\n\rml_dataset_id\x18\x03 \x01(\t\x12\x14\n\x0cprimary_keys\x18\x04 \x03(\t\x12M\n\x05query\x18\x05 \x01(\x0b\x32>.ai.h2o.featurestore.api.v1.OnlineMLDatasetRetrieveToken.Query\x12p\n\x18\x66\x65\x61ture_set_primary_keys\x18\x06 \x03(\x0b\x32N.ai.h2o.featurestore.api.v1.OnlineMLDatasetRetrieveToken.FeatureSetPrimaryKeys\x12\x15\n\ronline_schema\x18\x07 \x01(\t\x12 \n\x18online_pipeline_location\x18\x08 \x01(\t\x1a\x8f\x07\n\x05Query\x12\x61\n\x11selected_features\x18\x01 \x03(\x0b\x32\x46.ai.h2o.featurestore.api.v1.OnlineMLDatasetRetrieveToken.Query.Feature\x12Q\n\x04\x66rom\x18\x02 \x01(\x0b\x32\x43.ai.h2o.featurestore.api.v1.OnlineMLDatasetRetrieveToken.Query.From\x12Q\n\x04join\x18\x03 \x03(\x0b\x32\x43.ai.h2o.featurestore.api.v1.OnlineMLDatasetRetrieveToken.Query.Join\x1aP\n\x04\x46rom\x12\x16\n\x0e\x66\x65\x61ture_set_id\x18\x01 \x01(\t\x12!\n\x19\x66\x65\x61ture_set_major_version\x18\x02 \x01(\x05\x12\r\n\x05\x61lias\x18\x03 \x01(\t\x1a*\n\x07\x46\x65\x61ture\x12\x0e\n\x06prefix\x18\x01 \x01(\t\x12\x0f\n\x07\x66\x65\x61ture\x18\x02 \x01(\t\x1a\xfe\x03\n\x04Join\x12\x16\n\x0e\x66\x65\x61ture_set_id\x18\x01 \x01(\t\x12!\n\x19\x66\x65\x61ture_set_major_version\x18\x02 \x01(\x05\x12\r\n\x05\x61lias\x18\x03 \x01(\t\x12R\n\x02on\x18\x04 \x03(\x0b\x32\x46.ai.h2o.featurestore.api.v1.OnlineMLDatasetRetrieveToken.Query.Join.On\x12_\n\tjoin_type\x18\x05 \x01(\x0e\x32L.ai.h2o.featurestore.api.v1.OnlineMLDatasetRetrieveToken.Query.Join.JoinType\x1a\xb1\x01\n\x02On\x12T\n\x04left\x18\x01 \x01(\x0b\x32\x46.ai.h2o.featurestore.api.v1.OnlineMLDatasetRetrieveToken.Query.Feature\x12U\n\x05right\x18\x02 \x01(\x0b\x32\x46.ai.h2o.featurestore.api.v1.OnlineMLDatasetRetrieveToken.Query.Feature\"C\n\x08JoinType\x12\x1b\n\x17ONLINE_QUERY_JOIN_INNER\x10\x00\x12\x1a\n\x16ONLINE_QUERY_JOIN_LEFT\x10\x01\x1a\xf1\x01\n\x15\x46\x65\x61tureSetPrimaryKeys\x12\x16\n\x0e\x66\x65\x61ture_set_id\x18\x01 \x01(\t\x12!\n\x19\x66\x65\x61ture_set_major_version\x18\x02 \x01(\x05\x12n\n\x0bprimary_key\x18\x03 \x03(\x0b\x32Y.ai.h2o.featurestore.api.v1.OnlineMLDatasetRetrieveToken.FeatureSetPrimaryKeys.PrimaryKey\x1a-\n\nPrimaryKey\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x11\n\tdata_type\x18\x02 \x01(\tB\x10\x42\x0eOnlineApiProtob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ai.h2o.featurestore.api.v1.OnlineApi_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'B\016OnlineApiProto'
  _globals['_ONLINERETRIEVERESPONSE']._serialized_start=107
  _globals['_ONLINERETRIEVERESPONSE']._serialized_end=144
  _globals['_ONLINEINGESTIONREQUEST']._serialized_start=146
  _globals['_ONLINEINGESTIONREQUEST']._serialized_end=218
  _globals['_ONLINEINGESTIONRESPONSE']._serialized_start=220
  _globals['_ONLINEINGESTIONRESPONSE']._serialized_end=261
  _globals['_ONLINEINGESTIONTOKENREQUEST']._serialized_start=263
  _globals['_ONLINEINGESTIONTOKENREQUEST']._serialized_end=345
  _globals['_ONLINEINGESTIONTOKENRESPONSE']._serialized_start=347
  _globals['_ONLINEINGESTIONTOKENRESPONSE']._serialized_end=457
  _globals['_ONLINERETRIEVETOKENREQUEST']._serialized_start=459
  _globals['_ONLINERETRIEVETOKENREQUEST']._serialized_end=540
  _globals['_ONLINEMLDATASETRETRIEVETOKENREQUEST']._serialized_start=542
  _globals['_ONLINEMLDATASETRETRIEVETOKENREQUEST']._serialized_end=602
  _globals['_ONLINERETRIEVETOKENRESPONSE']._serialized_start=604
  _globals['_ONLINERETRIEVETOKENRESPONSE']._serialized_end=713
  _globals['_ONLINEFEATURESCHEMA']._serialized_start=715
  _globals['_ONLINEFEATURESCHEMA']._serialized_end=834
  _globals['_ONLINETOKEN']._serialized_start=837
  _globals['_ONLINETOKEN']._serialized_end=1210
  _globals['_ONLINERETRIEVETOKEN']._serialized_start=1213
  _globals['_ONLINERETRIEVETOKEN']._serialized_end=1485
  _globals['_ONLINERETRIEVETOKEN_PRIMARYKEY']._serialized_start=1440
  _globals['_ONLINERETRIEVETOKEN_PRIMARYKEY']._serialized_end=1485
  _globals['_ONLINEMLDATASETRETRIEVETOKEN']._serialized_start=1488
  _globals['_ONLINEMLDATASETRETRIEVETOKEN']._serialized_end=3034
  _globals['_ONLINEMLDATASETRETRIEVETOKEN_QUERY']._serialized_start=1879
  _globals['_ONLINEMLDATASETRETRIEVETOKEN_QUERY']._serialized_end=2790
  _globals['_ONLINEMLDATASETRETRIEVETOKEN_QUERY_FROM']._serialized_start=2153
  _globals['_ONLINEMLDATASETRETRIEVETOKEN_QUERY_FROM']._serialized_end=2233
  _globals['_ONLINEMLDATASETRETRIEVETOKEN_QUERY_FEATURE']._serialized_start=2235
  _globals['_ONLINEMLDATASETRETRIEVETOKEN_QUERY_FEATURE']._serialized_end=2277
  _globals['_ONLINEMLDATASETRETRIEVETOKEN_QUERY_JOIN']._serialized_start=2280
  _globals['_ONLINEMLDATASETRETRIEVETOKEN_QUERY_JOIN']._serialized_end=2790
  _globals['_ONLINEMLDATASETRETRIEVETOKEN_QUERY_JOIN_ON']._serialized_start=2544
  _globals['_ONLINEMLDATASETRETRIEVETOKEN_QUERY_JOIN_ON']._serialized_end=2721
  _globals['_ONLINEMLDATASETRETRIEVETOKEN_QUERY_JOIN_JOINTYPE']._serialized_start=2723
  _globals['_ONLINEMLDATASETRETRIEVETOKEN_QUERY_JOIN_JOINTYPE']._serialized_end=2790
  _globals['_ONLINEMLDATASETRETRIEVETOKEN_FEATURESETPRIMARYKEYS']._serialized_start=2793
  _globals['_ONLINEMLDATASETRETRIEVETOKEN_FEATURESETPRIMARYKEYS']._serialized_end=3034
  _globals['_ONLINEMLDATASETRETRIEVETOKEN_FEATURESETPRIMARYKEYS_PRIMARYKEY']._serialized_start=1440
  _globals['_ONLINEMLDATASETRETRIEVETOKEN_FEATURESETPRIMARYKEYS_PRIMARYKEY']._serialized_end=1485
# @@protoc_insertion_point(module_scope)
